﻿using System;
using System.Collections.Generic;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Threading.Tasks;

namespace TestClient
{
    /// <summary>
    /// 프로그램
    /// </summary>
    class Program
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region HTTP 클라이언트 구하기 - GetHTTPClient(baseAddress)

        /// <summary>
        /// HTTP 클라이언트 구하기
        /// </summary>
        /// <param name="baseAddress">기준 주소</param>
        /// <returns>HTTP 클라이언트</returns>
        private static HttpClient GetHTTPClient(string baseAddress)
        {
            HttpClient client = new HttpClient();

            client.BaseAddress = new Uri(baseAddress);

            client.DefaultRequestHeaders.Add("APIKey", "A7A54AD3-91E1-4C3A-98C6-A17E39C9EA3D");

            client.DefaultRequestHeaders.Accept.Clear();

            client.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));

            return client;
        }

        #endregion
        #region 리스트 구하기 - GetList(client)

        /// <summary>
        /// 리스트 구하기
        /// </summary>
        /// <param name="client">HTTP 클라이언트</param>
        /// <returns>리스트</returns>
        private static async Task<List<WeatherForecast>> GetList(HttpClient client)
        {
            string url = string.Format("/WeatherForecast");

            HttpResponseMessage response = await client.GetAsync(url);

            if(response.IsSuccessStatusCode)
            {
                List<WeatherForecast> list = await response.Content.ReadAsAsync<List<WeatherForecast>>();

                return list;
            }
            else
            {
                Console.WriteLine(response.StatusCode);
                Console.WriteLine(await response.Content.ReadAsStringAsync());
            }

            return null;
        }

        #endregion
        #region 프로그램 시작하기 - Main()

        /// <summary>
        /// 프로그램 시작하기
        /// </summary>
        static void Main()
        {
            using(HttpClient client = GetHTTPClient("https://localhost:44362"))
            {
                List<WeatherForecast> list = GetList(client).GetAwaiter().GetResult();

                for(int i = 0; i < list.Count; i++)
                {
                    Console.WriteLine(list[i].Summary);
                }
            }

            Console.WriteLine("--------------------------------------------------");
            Console.WriteLine("아무 키나 눌러 주시기 바랍니다.");
            Console.ReadKey(true);
        }

        #endregion
    }
}