﻿using Microsoft.AspNetCore.Http;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using System.Threading.Tasks;

namespace TestServer
{
    /// <summary>
    /// API 키 미들웨어
    /// </summary>
    public class APIKeyMiddleware
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// API 키명
        /// </summary>
        private static readonly string API_KEY_NAME = "APIKey";

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 다음 액션
        /// </summary>
        private readonly RequestDelegate nextAction;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - APIKeyMiddleware(nextAction)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="nextAction">다음 액션</param>
        public APIKeyMiddleware(RequestDelegate nextAction)
        {
            this.nextAction = nextAction;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 호출하기 (비동기) - InvokeAsync(context)

        /// <summary>
        /// 호출하기 (비동기)
        /// </summary>
        /// <param name="context">HTTP 컨텍스트</param>
        /// <returns>태스크</returns>
        public async Task InvokeAsync(HttpContext context)
        {
            if(!context.Request.Headers.TryGetValue(API_KEY_NAME, out var sourceAPIKey))
            {
                context.Response.StatusCode = 401;

                await context.Response.WriteAsync("API Key was not provided.");

                return;
            }

            IConfiguration configuration = context.RequestServices.GetRequiredService<IConfiguration>();

            string targetAPIKey = configuration.GetValue<string>(API_KEY_NAME);

            if(!targetAPIKey.Equals(sourceAPIKey))
            {
                context.Response.StatusCode = 401;

                await context.Response.WriteAsync("Unauthorized client.");

                return;
            }

            await nextAction(context);
        }

        #endregion
    }
}